package com.fossgalaxy.games.tbs.editor;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.editor.EditorView.MapToolListener;
import com.fossgalaxy.games.tbs.editor.EditorView.EditorModel;
import com.fossgalaxy.games.tbs.editor.EditorView.EditorView;
import com.fossgalaxy.games.tbs.editor.panels.*;
import com.fossgalaxy.games.tbs.editor.resources.StartingResourcesPanel;
import com.fossgalaxy.games.tbs.editor.tools.*;
import com.fossgalaxy.games.tbs.io.SettingsIO;
import com.fossgalaxy.games.tbs.io.map2.MapDef;
import com.fossgalaxy.games.tbs.parameters.GameSettings;
import com.fossgalaxy.games.tbs.ui.MouseController;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.File;

/**
 * Hello world!
 */
public class Editor {
    private final StartingResourcesPanel resources;

    private NewAction newAction;
    private SaveAction saveAction;
    private OpenAction openAction;

    public Editor(GameSettings settings, EditorModel editorModel){
        this.resources = new StartingResourcesPanel(settings, editorModel);

        SettingsIO io = new SettingsIO();

        this.newAction = new NewAction(editorModel);
        this.saveAction = new SaveAction(editorModel);
        this.openAction = new OpenAction(io, editorModel);
    }

    public static void main(String[] args) {

        String levelName = "map2.json";
        String gameName = "game.json";

        if (args.length != 0 && args.length != 2) {
            System.err.println("usage: [game] [level]");
            System.exit(1);
        }

        if (args.length == 2) {
            gameName = args[0];
            levelName = args[1];
        }

        //create the initial game state
        SettingsIO io = new SettingsIO();
        run(io, gameName, levelName);
    }

    public static void run(SettingsIO io, String gameName, String levelName) {

        GameSettings settings = io.loadSettings(gameName);
        settings.finish(io);

        EditorModel editorModel = new EditorModel(settings);
        Editor editor = new Editor(settings, editorModel);

        File levelFile = new File(levelName);
        if (levelFile.exists()) {
            MapDef mapDef = io.loadMapDef(levelName);
            editorModel.setMapDef(mapDef);
        }


        JFrame frame = new JFrame("CE810 editor");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setPreferredSize(new Dimension(800,600));

        JMenuBar bar = new JMenuBar();

        JMenu menu = new JMenu("tools");

        JMenuItem item = new JMenuItem("Starting Resources");
        item.addActionListener(actionEvent -> {
            editor.resources.showPanel();
        });
        menu.add(item);

        bar.add(menu);

        frame.setJMenuBar(bar);


        JToolBar toolBar = new JToolBar();
        toolBar.setFloatable(false);

        //misc
        toolBar.add(editor.newAction);

        //file io
        toolBar.addSeparator();
        toolBar.add(editor.saveAction);
        toolBar.add(editor.openAction);


        frame.add(toolBar, BorderLayout.NORTH);

        MapToolListener listener = new MapToolListener(editorModel);

        //tools
        toolBar.addSeparator();
        toolBar.add(new ToolAction(editorModel, new SinglePlace()));
        toolBar.add(new ToolAction(editorModel, new FloodFill(editorModel)));
        toolBar.add(new ToolAction(editorModel, new DeleteTool()));

        toolBar.addSeparator();
        toolBar.add(new ToggleLocations(frame, editorModel));

        JSplitPane panel = new JSplitPane();
        panel.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        panel.setResizeWeight(0.25);

        JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.addTab("terrain", new TerrainSelector(settings.getTerrainTypes(), editorModel));
        tabbedPane.addTab("resources", new ResourceSelector(settings.getResouceTypes(), editorModel));
        tabbedPane.addTab("entity", new EntityPanel(settings.getEntityTypes(), editorModel));

        //entity properties
        EntityProperties properties = new EntityProperties();
        editorModel.addEditorListener(properties);
        tabbedPane.addTab("prop", properties);

        panel.setLeftComponent(tabbedPane);


        editorModel.addEditorListener(listener);
        EditorView view = new EditorView(editorModel);
        view.addMouseListener(new EditorMouseController(view, editorModel));

        JScrollPane viewport = new JScrollPane(view);
        viewport.setPreferredSize(new Dimension(800, 600));
        viewport.setBackground(Color.BLACK);

        panel.setRightComponent(viewport);

        frame.add(panel);

        frame.pack();
        frame.setVisible(true);
    }

}
